unit Main;

{-------------------------------------------------------------------------------

StartCaptor - Controls links and apps automatically started by Windows
Copyright (C) 2000 Adam Stiles.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, Registry, ComCtrls, ExtCtrls, ShellApi, StartupFolder,
  ComObj, ShlObj, ActiveX;

type
  TMainForm = class(TForm)
    StartList: TListView;
    TopPanel: TPanel;
    FollowingLabel: TLabel;
    BottomPanel: TPanel;
    DeleteButton: TButton;
    AboutButton: TButton;
    Panel1: TPanel;
    CloseButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure StartListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure DeleteButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure LoadList;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

const
  VERNUM = '0.6';

implementation

{$R *.DFM}
//------------------------------------------------------------------------------
procedure TMainForm.LoadList;
const
  HKCU_RUN_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Run';
  HKLM_RUN_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Run';
  HKCU_RUNONCE_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce';
  HKLM_RUNONCE_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce';
  HKCU_RUNONCEEX_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnceEx';
  HKLM_RUNONCEEX_KEY = 'SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnceEx'; 
  procedure FillFromReg(Root: HKEY; Key: String);
  var
    I: Integer;
    Reg: TRegistry;
    Item: TListItem;
    Names: TStringList;
    Desc, Name, Location: String;
  begin
    Reg := TRegistry.Create;
    try
      Reg.RootKey := Root;
      if (Reg.OpenKey(Key, False)) then
      begin
        Names := TStringList.Create;
        try
          Reg.GetValueNames(Names);
          for I := 0 to Names.Count - 1 do
          begin
            Name := Names[I];
            Desc := Reg.ReadString(Names[I]);
            if (Root = HKEY_LOCAL_MACHINE) then
              Location := 'HKLM\' + Key
            else
              Location := 'HKCU\' + Key;
            Item := StartList.Items.Add;
            Item.Caption := Name;
            Item.SubItems.Add(Desc);
            Item.SubItems.Add(Location);            
          end;
        finally
          Names.Free;
        end;
      end;
    finally
      Reg.Free;
    end;
  end;
  function PathFromShortcut(const FileName: String): String;
  var
    Unk: IUnknown;
    Link: IShellLink;
    PerFile: IPersistFile;
    Find: TWin32FindData;
    WCFileName: PWideChar;
    PCFileName: PChar;
  begin
    // there should be a lot more error checking in this one
    Unk := CreateComObject(CLSID_ShellLink);
    PerFile := Unk as IPersistFile;
    WCFileName := StringToOleStr(FileName);
    try
      PerFile.Load(WCFileName, STGM_READ);
      Link := Unk as IShellLink;
      PCFileName := StrAlloc(MAX_PATH);
      try
        Link.GetPath(PCFileName, MAX_PATH - 1, Find, SLGP_UNCPRIORITY);
        Result := String(PCFileName);
      finally
        StrDispose(PCFileName);
      end;
    finally
      SysFreeString(WCFileName);
    end;
  end;
  procedure FillFromFolder(Dir: String);
  var
    Search: TSearchRec;
    Err: Integer;
    Item: TListItem;
  begin
    Err := FindFirst(Dir + '\*.lnk', faAnyFile, Search);
    if (Err = 0) then
    begin
      while (Err = 0) do
      begin
        Item := StartList.Items.Add;
        Item.Caption := Copy(Search.Name, 1, Length(Search.Name) - 4);
        Item.SubItems.Add(PathFromShortCut(Dir + '\' + Search.Name));
        Item.SubItems.Add(Dir + '\' + Search.Name);
        Err := FindNext(Search);
      end;
      FindClose(Search);
    end;
  end;
begin

  StartList.Items.BeginUpdate;
  try
    StartList.Items.Clear;

    // check registry sections
    FillFromReg(HKEY_CURRENT_USER, HKCU_RUN_KEY);
    FillFromReg(HKEY_LOCAL_MACHINE, HKLM_RUN_KEY);
    FillFromReg(HKEY_CURRENT_USER, HKCU_RUNONCE_KEY);
    FillFromReg(HKEY_LOCAL_MACHINE, HKLM_RUNONCE_KEY);

    // check other file system stuff
    FillFromFolder(GetUserStartupFolder);
    FillFromFolder(GetCommonStartupFolder);
  finally
    StartList.Items.EndUpdate;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
var
  NCMetrics: TNonClientMetrics;
begin
  Caption := 'StartCaptor ' + VERNUM;

  // load default font
  NCMetrics.cbSize := sizeof(NCMetrics);
  if (SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NCMetrics, 0)) then
    Font.Handle := CreateFontIndirect(NCMetrics.lfMessageFont);

  LoadList;
end;
//------------------------------------------------------------------------------
procedure TMainForm.StartListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  // enable the delete button if there are items selected
  if (Change=ctState) then
    DeleteButton.Enabled := (StartList.SelCount <> 0);
end;
//------------------------------------------------------------------------------
procedure TMainForm.DeleteButtonClick(Sender: TObject);
var
  Item: TListItem;
  Loc: string;
  Reg: TRegistry;
begin
  if (MessageDlg('Are you sure you want to delete this item?',
    mtConfirmation, [mbYes, mbNo],0) = mrYes) then
  begin
    Item:=StartList.Selected;
    if Assigned(Item) then
    begin
      if (Item.SubItems.count > 1) then
      begin
        Loc := Item.Subitems[1];

        // if this is a file then we delete it, if its a registry string
        // then we delete it
        if (Copy(Loc, 1, 4) = 'HKCU') or (Copy(Loc, 1, 4) = 'HKLM') then
        begin
          Reg := TRegistry.Create;
          try
            if (Copy(Loc, 1, 4) = 'HKLM') then
              Reg.RootKey := HKEY_LOCAL_MACHINE
            else
              Reg.RootKey := HKEY_CURRENT_USER;
            Loc := Copy(Loc, 6, Length(Loc));
            if (Reg.OpenKey(Loc, False)) then
            begin
              Reg.DeleteValue(Item.Caption);
              Reg.CloseKey;
            end;
          finally
            Reg.Free;
          end;
        end
        else
          DeleteFile(Loc);
        LoadList;
      end;
    end;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StartList.Items.Clear;
end;
//------------------------------------------------------------------------------
procedure TMainForm.AboutButtonClick(Sender: TObject);
begin
  MessageDlg('StartCaptor' + #13#10 + 'Copyright (c) 2000 Adam Stiles. All '
    + 'Rights Reserved.' + #13#10#13#10
    + 'See http://www.xcaptor.org for updates.',
    mtInformation, [mbOK], 0);
end;
//------------------------------------------------------------------------------
procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;
//------------------------------------------------------------------------------
initialization
  CoInitialize(nil);
finalization
  CoUninitialize;
end.
